macroscript ObjectResizer
category:"BodyulCG"
tooltip:"Object Resizer"
(
	local ObjectResizer
	local isOpen = false
	local dialogPos
	
	rollout ObjectResizer "Object Resizer" width:255 height:95
	(
		local nodeCallback
		local selectedNode
		local lockUI = false
		
		groupBox grp1 "Absolute" pos:[5, 6] width:120 height:84
		spinner spnAbsX "X:" pos:[20, 25] width:94 height:16 range:[-1e+07, 1e+07, 0] type:#worldunits
		spinner spnAbsY "Y:" pos:[20, 45] width:94 height:16 range:[-1e+07, 1e+07, 0] type:#worldunits
		spinner spnAbsZ "Z:" pos:[20, 65] width:94 height:16 range:[-1e+07, 1e+07, 0] type:#worldunits 
		groupBox grp2 "Offset" pos:[130 ,5] width:120 height:85
		spinner spnOffsetX "X:" pos:[145, 25] width:94 height:16 range:[-1e+07, 1e+07, 0] type:#worldunits
		spinner spnOffsetY "Y:" pos:[145, 45] width:94 height:16 range:[-1e+07, 1e+07, 0] type:#worldunits
		spinner spnOffsetZ "Z:" pos:[145, 65] width:94 height:16 range:[-1e+07, 1e+07, 0] type:#worldunits

		fn getNodeSize n =
		(
			local obb = nodeLocalBoundingBox n
			local rot = inverse n.rotation.controller.value
			local p1 = obb[1] * rot
			local p2 = obb[2] * rot

			return (p2 - p1)
		)
		
		fn setNodeSize n size = 
		(
			n.scale = [1, 1, 1]
			
			local obb = nodeLocalBoundingBox n
			local rot = inverse n.rotation.controller.value
			local p1 = obb[1] * rot
			local p2 = obb[2] * rot
			
			n.scale.controller.value = size / (p2 - p1)
		)
			
		fn updateNodeSize offset:false =
		(
			if selectedNode != undefined then
			(
				if offset then
					setNodeSize selectedNode [spnAbsX.value + spnOffsetX.value, spnAbsY.value + spnOffsetY.value, spnAbsZ.value + spnOffsetZ.value]
				else
					setNodeSize selectedNode [spnAbsX.value, spnAbsY.value, spnAbsZ.value]
			)
		)
		
		fn updateUI =
		(		
			if selectedNode != undefined then
			(
				local size = getNodeSize selectedNode
				
				spnAbsX.enabled = spnAbsY.enabled = spnAbsZ.enabled = true
				spnAbsX.value = size.x
				spnAbsY.value = size.y
				spnAbsZ.value = size.z
				
				spnOffsetX.enabled = spnOffsetY.enabled = spnOffsetZ.enabled = true			
				spnOffsetX.value = 0
				spnOffsetY.value = 0
				spnOffsetZ.value = 0
			)
			else
			(
				spnAbsX.enabled = spnAbsY.enabled = spnAbsZ.enabled = false
				spnAbsX.value = 0
				spnAbsY.value = 0
				spnAbsZ.value = 0
				spnOffsetX.enabled = spnOffsetY.enabled = spnOffsetZ.enabled = false
			)
		)
		
		fn beginEdit =
			lockUI = true
		
		fn endEdit =
		(
			lockUI = false
			updateUI()
		)
		
		fn cbUpdateNode e n =
		(
			if selection.count == 1 then
				selectedNode = selection[1] 
			else
				selectedNode = undefined

			updateUI()
		)
		
		fn cbUpdateUI e n =
		(
			if not lockUI then
				updateUI()
		)
		
		on ObjectResizer open do
		(
			if dialogPos != undefined then
				SetDialogPos ObjectResizer dialogPos;

			nodeCallback = nodeeventcallback \
			selectionChanged:cbUpdateNode \
			added:cbUpdateNode \
			deleted:cbUpdateNode \
			controllerOtherEvent:cbUpdateUI \
			geometryChanged:cbUpdateUI
			
			if selection.count == 1 then
				selectedNode = selection[1]
			
			updateUI()
		)
		
		on spnAbsX buttondown do
			beginEdit()
		
		on spnAbsX changed val do
			updateNodeSize()

		on spnAbsX buttonup do
			endEdit()
		
		on spnAbsY buttondown do
			beginEdit()
		
		on spnAbsY changed val do
			updateNodeSize()

		on spnAbsY buttonup do
			endEdit()
		
		on spnAbsZ buttondown do
			beginEdit()
		
		on spnAbsZ changed val do
			updateNodeSize()

		on spnAbsZ buttonup do
			endEdit()

		on spnAbsZ changed val do
			updateNodeSize()	
		
		on spnOffsetX buttondown do
			beginEdit()
		
		on spnOffsetX changed val do
			updateNodeSize offset:true
		
		on spnOffsetX buttonup do
			endEdit()
		
		on spnOffsetY buttondown do
			beginEdit()
		
		on spnOffsetY changed val do
			updateNodeSize offset:true
		
		on spnOffsetY buttonup do
			endEdit()
		
		on spnOffsetZ buttondown do
			beginEdit()
		
		on spnOffsetZ changed val do
			updateNodeSize offset:true
		
		on spnOffsetZ buttonup do
			endEdit()
		
		on ObjectResizer close do
		(
			nodeCallback = undefined
			dialogPos = getDialogPos ObjectResizer
			isOpen = false
			
			updateToolbarButtons()
			gc light:true
		)
	)
	
	on execute do
	(
		if isOpen then
		(
			destroyDialog ObjectResizer
			isOpen = false
		)
		else
		(
			createDialog ObjectResizer
			isOpen = true
		)	
	)
	
	on isChecked do
		isOpen;

	on closeDialogs do
		destroyDialog ObjectResizer;
)
	